@@ -21,8 +21,8 @@
  *
  * CVS Information:
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager.h,v $
- * $Date: 2012/03/27 15:19:02 $
- * $Revision: 1.70.2.5 $
+ * $Date: 2012/06/18 16:44:17 $
+ * $Revision: 1.70.2.9 $
  * $Author: bester $
  */
 #ifndef GLOBUS_GRAM_JOB_MANAGER_INCLUDE
@@ -413,6 +413,19 @@ typedef struct globus_i_gram_usage_job_tracker_s
     char *                              user_dn;
 } globus_i_gram_usage_job_tracker_t;
 
+typedef struct
+{
+    /** Address of the client which submitted the job (hashtable key) */
+    char *                              client_addr;
+    /** Queue of script contexts ready to run */
+    globus_priority_q_t                 script_queue;
+    /** Number of script slots available for running scripts */
+    int                                 script_slots_available;
+    /** Fifo of available script handles */
+    globus_fifo_t                       script_handles;
+}
+globus_gram_job_manager_scripts_t;
+
 /**
  * Runtime state for a LRM instance. All of these items are
  * computed from the configuration state above and may change during the
@@ -471,13 +484,12 @@ typedef struct globus_gram_job_manager_s
     char *                              lock_path;
     /** Pid file path */
     char *                              pid_path;
-
-    /** Queue of script contexts ready to run */
-    globus_priority_q_t                 script_queue;
-    /** Number of script slots available for running scripts */
-    int                                 script_slots_available;
-    /** Fifo of available script handles */
-    globus_fifo_t                       script_handles;
+    /** OSG wants to have different clients connecting to the same job manager
+     * to have separate script queues fto have scalability with nonresponsive
+     * clients. We hash on client's address and have separate script queue and
+     * available slots.
+     */
+    globus_list_t *                     scripts_per_client;
     /** Fifo of job state callback contexts to run */
     globus_fifo_t                       state_callback_fifo;
     /** Number of job state contact slots available */
@@ -515,7 +527,11 @@ typedef struct globus_gram_job_manager_s
      * Periodic callback handle to abort if something removes the lock file.
      */
     globus_callback_handle_t            lockcheck_handle;
-    
+
+    /**
+     * Periodic callback handle to clse idle perl script xio handles
+     */
+    globus_callback_handle_t            idle_script_handle;
 }
 globus_gram_job_manager_t;
 
@@ -1359,6 +1375,9 @@ void
 globus_gram_job_manager_script_close_all(
     globus_gram_job_manager_t *         manager);
 
+void
+globus_gram_script_close_idle(
+    void *                              arg);
 
 extern globus_xio_driver_t              globus_i_gram_job_manager_popen_driver;
 extern globus_xio_stack_t               globus_i_gram_job_manager_popen_stack;
@@ -1545,6 +1564,9 @@ globus_gram_split_subjobs(
     const char *                        job_id,
     globus_list_t **                    subjobs);
 
+int
+globus_i_gram_mkdir(
+    char *                              path);
 /* globus_gram_job_manager_usagestats.c */
 
 globus_result_t
