@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager_config.c,v $
- * $Date: 2012/06/06 14:04:51 $
- * $Revision: 1.24.4.3 $
+ * $Date: 2012/08/17 14:37:15 $
+ * $Revision: 1.24.4.4 $
  * $Author: bester $
  */
 
@@ -787,7 +787,7 @@ globus_gram_job_manager_config_destroy(
     }
     if (config->extra_envvars)
     {
-        free(config->extra_envvars);
+        globus_list_destroy_all(config->extra_envvars, free);
     }
     if (config->seg_module)
     {
